# -*- coding: utf8 -*-

from datetime import date, timedelta

from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import PermissionDenied
from django.shortcuts import render, get_object_or_404, redirect
from django.views.generic.detail import DetailView
from django.views.generic.edit import FormView
from django_tables2 import SingleTableMixin, SingleTableView

from lib.access_auth import access_restricted, permission_needed
from lib.tools import confirmation
from portailuser.models import PortailUser
from .forms import ChoixForm, SondageForm, VotantsForm
from .models import Sondage, Voix
from .tables import ResultatsTable, VotantsTable


@access_restricted
def accueil(request):
    """Récupère et affiche tous les sondages, démarrés ou non, auxquels participe l'utilisateur."""
    maintenant = date.today()
    Sondage.objects.filter(date_fin__lte=maintenant - timedelta(days=30)).delete()

    c = {
        "sondages_perso": Sondage.objects.filter(auteur=request.user),
        "sondages_ouverts": request.user.sondage_set.filter(est_demarre=True, date_fin__gt=maintenant, supprime=False),
        "sondages_clotures": request.user.sondage_set.filter(
            est_demarre=True, date_fin__lte=maintenant, supprime=False
        ),
    }

    return render(request, "sondages/accueil.html", c)


class SondageBaseView(LoginRequiredMixin, DetailView):
    context_object_name = "sondage"
    queryset = Sondage.objects.filter(est_demarre=True, supprime=False)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        sondage = self.object
        est_cloture = sondage.date_fin <= date.today()
        context["est_cloture"] = est_cloture
        context["peut_voir_res"] = not sondage.resultats_masques or sondage.auteur == self.request.user

        try:
            vote = Voix.objects.get(sondage=sondage, votant=self.request.user)
            context["deja_vote"] = vote.exprimee
            # Les sondages anonymes sont non modifiables
            context["peut_voter"] = not sondage.anonyme or (sondage.anonyme and not vote.exprimee)
        except Voix.DoesNotExist:
            context["peut_voter"] = False
        return context


class VoteView(SondageBaseView):
    template_name = "sondages/vote.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        sondage = self.object

        choix_sondage = sondage.choix_set.order_by("id")
        vote_user = Voix.objects.filter(sondage=sondage, votant=self.request.user)
        if vote_user.exists():
            choix_user = vote_user[0].choix.all()
            for choix in choix_sondage:
                if choix in choix_user:
                    choix.user_favorable = True

        context["choix_sondage"] = choix_sondage
        return context

    def post(self, request, *args, **kwargs):
        sondage = self.get_object()
        vote = request.POST.getlist("choix_vote")
        sondage.set_vote(request.user, vote)
        return redirect("sondages:resultats", slug=sondage.slug)


class ResultatView(SondageBaseView):
    template_name = "sondages/resultats.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        sondage = self.object

        if not context["peut_voir_res"]:
            raise PermissionDenied

        choix_sondage = sondage.choix_set.order_by("id")
        choix_dict = dict([(c.id, c) for c in choix_sondage])
        resultats = Voix.objects.filter(sondage=sondage, exprimee=True)
        for voix in resultats:
            favorables = voix.choix.all()
            voix.sel_favorable = [choix in favorables for choix in choix_sondage]
            for choix in favorables:
                choix_dict[choix.id].votes += 1

        context["choix_sondage"] = choix_sondage
        return context


class ResultatListView(LoginRequiredMixin, SingleTableView):
    table_class = ResultatsTable
    template_name = "sondages/resultats_list.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        sondage = get_object_or_404(Sondage, slug=self.kwargs["slug"])
        context["sondage"] = sondage

        if sondage.resultats_masques and sondage.auteur != self.request.user:
            raise PermissionDenied
        return context

    def get_queryset(self):
        return Voix.objects.filter(sondage__slug=self.kwargs["slug"], exprimee=True).order_by(
            "votant__last_name", "votant__first_name"
        )


@access_restricted
def edition_infos(request, slug=None):
    if slug is None:
        sondage = None
    else:
        sondage = get_object_or_404(Sondage, slug=slug, auteur=request.user, est_demarre=False)

    if request.method == "POST":
        form = SondageForm(request.POST, instance=sondage)
        if form.is_valid():
            sondage = form.save(commit=False)
            sondage.auteur = request.user
            sondage.save()
            return redirect("sondages:edition_choix", slug=sondage.slug)
    else:
        form = SondageForm(instance=sondage)

    return render(request, "sondages/edition_infos.html", locals())


@access_restricted
def edition_choix(request, slug):
    sondage = get_object_or_404(Sondage, slug=slug, auteur=request.user, est_demarre=False)

    if request.method == "POST":
        form = ChoixForm(sondage, request.POST)
        if form.is_valid() and not form.bouton_presse:
            donnees = form.donnees_entrees()
            choix = []
            for element in donnees:
                element = element.strip()
                if element and element not in choix:
                    choix.append(element)

            if len(choix) >= 2:
                sondage.set_choix(choix)
                return redirect("sondages:edition-votants", slug=sondage.slug)
            else:
                erreur = True

    else:
        form = ChoixForm(sondage)

    return render(request, "sondages/edition_choix.html", locals())


class EditionVotantsView(LoginRequiredMixin, SingleTableMixin, FormView):
    form_class = VotantsForm
    table_class = VotantsTable
    template_name = "sondages/edition_votants.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        sondage = get_object_or_404(Sondage, slug=self.kwargs["slug"], auteur=self.request.user, est_demarre=False)
        context["sondage"] = sondage
        context["has_votants"] = sondage.votants.exists()
        return context

    def get_queryset(self):
        return Voix.objects.filter(sondage__slug=self.kwargs["slug"]).order_by(
            "votant__last_name", "votant__first_name"
        )

    def form_invalid(self, form):
        messages.error(self.request, "Aucun utilisateur trouvé.")
        return super().form_invalid(form)

    def form_valid(self, form):
        sondage = get_object_or_404(Sondage, slug=self.kwargs["slug"], auteur=self.request.user, est_demarre=False)
        sondage.ajouter_votant(form.cleaned_data["user"])
        return redirect("sondages:edition-votants", slug=self.kwargs["slug"])


@access_restricted
def ajouter_votants(request, slug):
    """Ajouter un votant."""
    sondage = get_object_or_404(Sondage, slug=slug, auteur=request.user, est_demarre=False)

    if request.method == "POST":
        form = VotantsForm(request.POST)
        if form.is_valid():
            uid = form.data["uid"]
            utilisateur = PortailUser.objects.get_from_ldap(uid)
            sondage.ajouter_votant(utilisateur)

    return redirect("sondages:choix-votants", slug=slug)


@access_restricted
def supprimer_votants(request, slug):
    sondage = get_object_or_404(Sondage, slug=slug, auteur=request.user, est_demarre=False)

    if request.method != "POST":
        message = "Êtes-vous sûr de vouloir supprimer les votants de ce sondage ?"
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer les votants")]
        return confirmation(request, message, boutons, "Suppression des votants")

    if "conf_ok" in request.POST:
        sondage.supprimer_votants()

    return redirect("sondages:edition-votants", slug=slug)


@access_restricted
def demarrer_sondage(request, slug):
    sondage = get_object_or_404(Sondage, slug=slug, auteur=request.user, est_demarre=False)

    if request.method == "POST":
        sondage.est_demarre = True
        sondage.date_debut = date.today()
        sondage.save()

    return redirect("sondages:vote", slug=sondage.slug)


@access_restricted
def supprimer(request, slug=None):
    sondage = get_object_or_404(Sondage, slug=slug, auteur=request.user)

    if request.method != "POST":
        message = "Êtes-vous sûr de vouloir supprimer ce sondage ?"
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer ce sondage")]
        return confirmation(request, message, boutons, "Suppression d’un sondage")

    if "conf_ok" in request.POST:
        sondage.supprime = True
        sondage.save()

    return redirect("sondages:accueil")


@access_restricted
def remettre(request, slug=None):
    sondage = get_object_or_404(Sondage, slug=slug, auteur=request.user)

    if request.method != "POST":
        message = "Êtes-vous sûr de vouloir remettre ce sondage ?"
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Remettre ce sondage")]
        return confirmation(request, message, boutons, "Remise d’un sondage")

    if "conf_ok" in request.POST:
        sondage.supprime = False
        sondage.save()

    return redirect("sondages:accueil")


@permission_needed("gerer_sondages")
def supprimer_admin(request, slug=None):
    sondage = get_object_or_404(Sondage, slug=slug)

    if request.method != "POST":
        message = "Êtes-vous sûr de vouloir supprimer ce sondage ?"
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer ce sondage")]
        return confirmation(request, message, boutons, "Suppression d’un sondage")

    if "conf_ok" in request.POST:
        sondage.supprime = True
        sondage.save()

    return redirect("gestion:sondages:liste")


@permission_needed("gerer_sondages")
def remettre_admin(request, slug=None):
    sondage = get_object_or_404(Sondage, slug=slug)

    if request.method != "POST":
        message = "Êtes-vous sûr de vouloir remettre ce sondage ?"
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Remettre ce sondage")]
        return confirmation(request, message, boutons, "Remise d’un sondage")

    if "conf_ok" in request.POST:
        sondage.supprime = False
        sondage.save()

    return redirect("gestion:sondages:liste")


@permission_needed("gerer_sondages")
def liste(request):
    maintenant = date.today()
    Sondage.objects.filter(date_fin__lte=maintenant - timedelta(days=30)).delete()
    Sondage.objects.filter(date_fin__lte=maintenant - timedelta(days=7), supprime=True).delete()

    c = {
        "sondages": Sondage.objects.filter(supprime=False),
        "sondages_supprimes": Sondage.objects.filter(supprime=True),
    }

    return render(request, "sondages/liste.html", c)


@permission_needed("gerer_sondages")
def suppression_definitive(request, slug=None):
    sondage = get_object_or_404(Sondage, slug=slug)

    if request.method != "POST":
        message = "Êtes-vous sûr de vouloir supprimer définitivement ce sondage ?"
        boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer ce sondage")]
        return confirmation(request, message, boutons, "Suppression d’un sondage")

    if "conf_ok" in request.POST:
        sondage.delete()

    return redirect("gestion:sondages:liste")


@permission_needed("gerer_sondages")
def infos(request, slug):
    sondage = get_object_or_404(Sondage, slug=slug)

    maintenant = date.today()
    est_cloture = sondage.date_fin <= maintenant

    choix_sondage = sondage.choix_set.order_by("id")

    votants = sondage.votants.order_by("last_name", "first_name")

    c = {
        "sondage": sondage,
        "est_cloture": est_cloture,
        "choix_sondage": choix_sondage,
        "votants": votants,
    }
    return render(request, "sondages/infos.html", c)
